//
//
//  WARNING!!!!
//  DO NOT LOAD THIS CODE INTO THE '4618
//  CHECK YOUR JTAG CABLE CONNECTION NOW!
//
//
//******************************************************************************
//  Lab 6 '2013 USI code - I2C Slave Transmitter
//
//  Instructions:
//   Load this code into the '2013 chip on the board.  Run it, then unplug
//   the JTAG cable and plug it into the header for the '4618.
//
//  Description: I2C Master communicates with 'F2013 I2C Slave transmitter
//  Temerature sensor 16-bit result is transmitted to the
//  I2C master. The SD16_A measures temperature after each query by the master.
//  Timer A is used to delay each conversion until VREF has settled.  LED on
//  P1.0 pulses during Vref settle. CPU remains in LPM4 until I2C active.
//
//  ACLK = N/A, MCLK = SMCLK = Cal 1MHz
//
//                  Slave
//               MSP430F2013
//             -----------------
//            |              XIN|-
//            |                 |
//            |             XOUT|-
//            |                 |
//      LED <-|P1.0             |           Master
//            |                 |
//            |         SDA/P1.7|<------>|I2C DATA
//            |         SCL/P1.6|<-------|I2C CLOCK
//
//  Note: internal pull-ups on Slave are used in this example for SDA & SCL
//
//******************************************************************************
#include  "msp430x20x3.h"

char SLV_Addr = 0x90;                   // Address is 0x48<<1 for R/W
int I2C_State = 0;                      // State variable
int byte_cnt = 2;                       // Bytes to TX
unsigned int ADC_result = 0;            // Store ADC result

void main(void)
{
  WDTCTL = WDTPW + WDTHOLD;             // Stop watchdog

  BCSCTL1 = CALBC1_1MHZ;                // Set DCO
  DCOCTL = CALDCO_1MHZ;

  SD16CTL = SD16SSEL_1;                 // SMCLK
  SD16INCTL0 = SD16INCH_6;              // A6+/- Temperature sensor
  SD16CCTL0 = SD16SNGL + SD16IE ;       // Single conv, interrupt

  P1OUT = 0xC0;                         // P1.6 & P1.7 Pullups
  P1REN |= 0xC0;                        // P1.6 & P1.7 Pullups
  P1DIR = 0xFF;                         // All P1.x output
  P2OUT = 0x00;                         // All P2.x low
  P2DIR = 0xFF;                         // All P2.x output
  P2SEL &= ~0xC0;                       // P2.6&7 GPIO

  USICTL0 = USIPE6+USIPE7+USISWRST;     // Port & USI mode setup
  USICTL1 = USII2C+USIIE+USISTTIE;      // Enable I2C mode & USI interrupts
  USICKCTL = USICKPL;                   // Setup clock polarity
  USICNT |= USIIFGCC;                   // Disable automatic clear control
  USICTL0 &= ~USISWRST;                 // Enable USI
  USICTL1 &= ~USIIFG;                   // Clear pending flag

  TACCTL0 = CCIE;                       // CCR0 interrupt enabled
  TACCR0 = 5000;                        // ~5msec at 1MHz TA clock

  byte_cnt = 2;                         // Define # of bytes to TX

  while(1)
  {
    P1OUT |= 0x01;                      // LED On
    SD16CTL |= SD16REFON;               // 1.2V ref on
    TACTL = TASSEL_2 + MC_1;            // SMCLK, up mode
    _BIS_SR(LPM0_bits + GIE);           // Enter LPM0 w/ interrupt

    _NOP();
  }
}

// USI interrupt service routine & I2C state machine
#pragma vector = USI_VECTOR
__interrupt void USI_TXRX (void)
{
  if (USICTL1 & USISTTIFG)              // Start entry?
  {
    I2C_State = 2;                      // Enter 1st state on start
  }

  switch(__even_in_range(I2C_State,12))
    {
      case 0: //Idle, should not get here
              break;

      case 2: //RX Address
              USICNT = (USICNT & 0xE0) + 0x08; // Bit counter = 8, RX Address
              USICTL1 &= ~USISTTIFG;    // Clear start flag
              I2C_State = 4;            // Go to next state: check address
              break;

      case 4: // Process Address and send (N)Ack
              if (USISRL & 0x01)        // If read...
                SLV_Addr++;             // Save R/W bit
              USICTL0 |= USIOE;         // SDA = output
              if (USISRL == SLV_Addr)   // Address match?
              {
                USISRL = 0x00;          // Send Ack
                I2C_State = 8;          // Go to next state: TX data
              }
              else
              {
                USISRL = 0xFF;          // Send NAck
                I2C_State = 6;          // Go to next state: prep for next Start
              }
              USICNT |= 0x01;           // Bit counter = 1, send (N)Ack bit
              break;

      case 6: // Prep for Start condition
              USICTL0 &= ~USIOE;        // SDA = input
              SLV_Addr = 0x90;          // Reset slave address
              I2C_State = 0;            // Reset state machine
              break;

      case 8: // Send Data byte
              USICTL0 |= USIOE;         // SDA = output
              USISRL = ADC_result;      // Send 1st data byte (MSB)
              byte_cnt--;
              USICNT |=  0x08;          // Bit counter = 8, TX data
              I2C_State = 10;           // Go to next state: receive (N)Ack
              break;

      case 10:// Receive Data (N)Ack
              USICTL0 &= ~USIOE;        // SDA = input
              USICNT |= 0x01;           // Bit counter = 1, receive (N)Ack
              I2C_State = 12;           // Go to next state: check (N)Ack
              break;

      case 12:// Process Data Ack/NAck, Handle additional bytes, if any
              if (USISRL & 0x01)        // If Nack received...
              {
                if (byte_cnt == 0);
                //... User code here
              }
              else                      // Ack received
              {
                if (byte_cnt > 0)
                {
                  USICTL0 |= USIOE;     // SDA = output
                  USISRL = ADC_result >> 8; // Send next data byte (LSB)
                  byte_cnt--;
                  USICNT |=  0x08;      // Bit counter = 8, TX data
                  I2C_State = 10;       // Go to next state: receive (N)Ack
                  break;
                }
              }
              // Prep for Start condition
              USICTL0 &= ~USIOE;        // SDA = input
              SLV_Addr = 0x90;          // Reset slave address
              I2C_State = 0;            // Reset state machine
              byte_cnt = 2;
              _BIC_SR_IRQ(LPM4_bits);   // Exit from LPM4 on RETI
              break;
    }

  USICTL1 &= ~USIIFG;                   // Clear pending flags
}

// SD16_A interrupt service routine
#pragma vector=SD16_VECTOR
__interrupt void SD16ISR(void)
{
  SD16CTL &= ~SD16REFON;                // 1.2V ref off
  ADC_result = SD16MEM0;                // Load result for TX
  ADC_result = __swap_bytes(ADC_result);
  _BIS_SR_IRQ(LPM4_bits);               // Exit into low power mode 4
}

// Timer A0 interrupt service routine
#pragma vector=TIMERA0_VECTOR
__interrupt void Timer_A (void)
{
  TACTL = TACLR;                        // Stop and clear Timer_A
  P1OUT &= ~0x01;                       // LED Off
  SD16CCTL0 |= SD16SC;                  // Start SD16_A conversion
}
